/**************************************************************************
**   HELP.C
**
**   NetWare 3 file server online help. Illustrates how to add a console
**   command on NetWare file servers.
**
**   Copyright 1996 Novell, Inc.
**
**   DISCLAIMER  
**  
**   Novell, Inc. makes no representations or warranties with respect
**   to this program, and specifically disclaims any express or
**   implied warranties of merchantability, title, or fitness for a
**   particular purpose.
**
**   This program may be distributed in source code or executable forms,
**   provided the program is distributed free of charge (or a nominal
**   charge made for duplication and and distribution costs), and all
**   copyright notices are left intact.
**   
**   Novell is not responsible for lost profits or revenue, loss of use
**   of the software, loss of data, costs of re-creating lost data, the
**   cost of any substitute equipment or program, or claims by any party
**   other than you.  Novell strongly recommends a backup be made before
**   any software is installed.   Technical support for this software
**   may be provided at the discretion of Novell.
**
#include <stdio.h>
#include <string.h>
#include <conio.h>
#include <errno.h>
#include <nwdir.h>
#include <process.h>
#include <advanced.h>
#include <nwenvrn.h>
#include "help.h"

//Global data. Used for communication between the command interpreter and the
//command handler thread
char    helpCommand[CMD_LENGTH] = "\0";
int     majorVersion, minorVersion;
char    command[5];

/*
	The console command interpreter function.
	The program registers this function with the NetWare OS. When the user types
	a command that the server's internal command interpreter does not recognize,
	it will pass the command line to this function, which can then decide whether
	the command line contains a command it wants to handle. Multiple console
	command interpreters can therefore be registered with the OS. Each one must
	return 0 if it handled the command passed to it, or 1 if not.
*/

static LONG CommandLineInterpreter(LONG screenID, BYTE *commandLine)
{
	/*
		Figure out whether this is a command we're interested in, then schedule
		the thread to handle the request. You may have one thread to process all
		commands recognized by the program, or one thread for each command. If
		you use multiple threads, then this function would schedule the thread
		that processes the appropriate command.

		This function should execute quickly and return control to the server's
		internal console command interpreter. So it's a good idea to have separate
		thread(s) to process console commands, and just have this function identify
		the commands we're interested, wake the appropriate thread to process the
		command, then return. If you want to do all of the work here, you must change
		the CLIB context to that of a thread group in your NLM. You can use
		SetThreadGroupID to do this.  Remember to switch it back before you return.
	*/

	char    *pCommand;

	if( !strnicmp(command, (char *)commandLine, 4) )
	{
		if (strlen((char *)commandLine) > 4)
		{
			pCommand = (char *)(commandLine + 5);
			memcpy(helpCommand, pCommand, CMD_LENGTH);
		}
		else
			memset(helpCommand, 0, CMD_LENGTH);
		ScheduleHelpCommand();
		return HANDLEDCOMMAND;
	}
	return NOTMYCOMMAND;
}

/*  Structure used to register/deregister a console handler with the OS */
static  struct  commandParserStructure
	commandParser = {0, CommandLineInterpreter, 0};

/*  The following function is called during NLM shutdown */
static void RemoveConsoleHandler()
{
	UnRegisterConsoleCommand(&commandParser);
}

void main(void)
{
	unsigned int    nlmHandle;
	FILE_SERV_INFO  serverInfo;

	if (GetServerInformation(50, &serverInfo))
	{
		ConsolePrintf("\n\r  HELP.NLM: Unable to get server version number\n\r");
		exit(1);
	}

	majorVersion = serverInfo.netwareVersion;
	minorVersion = serverInfo.netwareSubVersion;
	if (majorVersion == 3)
		strcpy(command, "HELP");
	else
		strcpy(command, "WHAT");

	ConsolePrintf("\n\r  Example program for Novell Developer Notes\n\r");
	ConsolePrintf("  Produced by Novell Systems Research Department\n\r");
	ConsolePrintf("  Distribute freely.\n\r");
	ConsolePrintf("  Running on NetWare v%d.%d\n\r", majorVersion, minorVersion);
	ConsolePrintf("  Adding %s console command\n\n\r", command);

	nlmHandle = GetNLMHandle();
	if (!(commandParser.RTag = (struct ResourceTagStructure *)AllocateResourceTag(nlmHandle, (BYTE *)"Help Console Command", ConsoleCommandSignature)))
	{
		ConsolePrintf("HELP: Unable to allocate resource tag.\n\r");
		exit(1);
	}

	if (BeginThread(HandleHelpRequest, NULL, 8192, NULL) == EFAILURE)
	{
		ConsolePrintf("HELP: Couldn't start HandleHelpRequest thread.\n\r");
		exit(1);
	}

	ThreadSwitch();
	RegisterConsoleCommand(&commandParser);
	atexit(RemoveConsoleHandler);
	ExitThread(EXIT_THREAD,0);
}
